/* copyright (c) 2013-2015 vincent forest (vaplv@free.fr)
 *
 * the rsys library is free software: you can redistribute it and/or modify
 * it under the terms of the gnu lesser general public license as published
 * by the free software foundation, either version 3 of the license, or
 * (at your option) any later version.
 *
 * the rsys library is distributed in the hope that it will be useful,
 * but without any warranty; without even the implied warranty of
 * merchantability or fitness for a particular purpose. see the
 * gnu lesser general public license for more details.
 *
 * you should have received a copy of the gnu lesser general public license
 * along with the rsys library. if not, see <http://www.gnu.org/licenses/>. */

#include "stretchy_array.h"
#include "math.h"

static void
test_double(void)
{
  double* array = NULL;

  CHECK(sa_size(array), 0);
  sa_clear(array);

  CHECK(sa_add(array, 4), array);
  CHECK(sa_size(array), 4);

  array[0] = PI;
  array[1] = PI;
  array[2] = PI;
  array[3] = PI;
  CHECK(sa_last(array), PI);

  sa_push(array, 1.2345);
  CHECK(sa_last(array), 1.2345);
  sa_push(array, 6.7890);
  CHECK(sa_last(array), 6.7890);

  CHECK(array[0], PI);
  CHECK(array[1], PI);
  CHECK(array[2], PI);
  CHECK(array[3], PI);
  CHECK(array[4], 1.2345);
  CHECK(array[5], 6.7890);
  CHECK(sa_size(array), 6);

  sa_clear(array);
  CHECK(sa_size(array), 0);

  sa_release(array);
}

static void
test_int(void)
{
  int* array = NULL;
  int* p;
  size_t i;

  CHECK(sa_size(array), 0);

  FOR_EACH(i, 0, 1024)
    sa_push(array, (int)i);

  CHECK(sa_size(array), 1024);

  FOR_EACH(i, 0, 1024)
    CHECK(array[i], (int)i);

  CHECK(sa_last(array), 1023);

  p = sa_add(array, 32);
  CHECK(sa_size(array), 1056);

  FOR_EACH(i, 0, 32)
    p[i] = (int)i;

  CHECK(sa_last(array), 31);
  FOR_EACH(i, 0, 32)
    array[1024 + i] = (int)i;

  sa_clear(array);
  CHECK(sa_size(array), 0);
  p = sa_add(array, 16);
  CHECK(p, array);
  CHECK(sa_size(array), 16);
  FOR_EACH(i, 0, 16)
    p[i] = -(int)i;

  FOR_EACH(i, 0, 16)
    CHECK(array[i], -(int)i);

  sa_release(array);
}

int
main(int argc, char** argv)
{
  (void)argc, (void)argv;
  test_int();
  test_double();
  CHECK(mem_allocated_size(), 0);
  return 0;
}
