/* Copyright (C) |Meso|Star> 2015-2017 (contact@meso-star.com)
 *
 * This software is a program whose purpose is to test the spp library.
 *
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software. You can use,
 * modify and/or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * As a counterpart to the access to the source code and rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty and the software's author, the holder of the
 * economic rights, and the successive licensors have only limited
 * liability.
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading, using, modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean that it is complicated to manipulate, and that also
 * therefore means that it is reserved for developers and experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and, more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms. */

#include "ssp.h"
#include "test_ssp_utils.h"

#define NBS 1000000

int
main(int argc, char** argv)
{
  struct ssp_rng* rng;
  struct mem_allocator allocator;
  struct ssp_ranst_piecewise_linear *pwl;
  int i;
  double exp_mean = 5, mean;
  double exp_std = 10 / sqrt(12) /*sqrt((b - a)² / 12) */, std;
  double x = 0, x2 = 0;
  const double intervals[] = { 0, 1, 3, 5, 7, 8, 10 };
  const double weights[] = { 1, 1, 1, 1, 1, 1, 1 };
  (void)argc, (void)argv;

  mem_init_proxy_allocator(&allocator, &mem_default_allocator);

  CHECK(ssp_rng_create(&allocator, &ssp_rng_mt19937_64, &rng), RES_OK);

  CHECK(ssp_ranst_piecewise_linear_create(NULL, NULL), RES_BAD_ARG);
  CHECK(ssp_ranst_piecewise_linear_create(NULL, &pwl), RES_OK);
  CHECK(ssp_ranst_piecewise_linear_ref_put(pwl), RES_OK);

  CHECK(ssp_ranst_piecewise_linear_create(&allocator, NULL), RES_BAD_ARG);
  CHECK(ssp_ranst_piecewise_linear_create(&allocator, &pwl), RES_OK);

  CHECK(ssp_ranst_piecewise_linear_setup
    (NULL, intervals, weights, sizeof(intervals)/sizeof(double)), RES_BAD_ARG);
  CHECK(ssp_ranst_piecewise_linear_setup
    (pwl, NULL, weights, sizeof(intervals)/sizeof(double)), RES_BAD_ARG);
  CHECK(ssp_ranst_piecewise_linear_setup
    (pwl, intervals, NULL, sizeof(intervals)/sizeof(double)), RES_BAD_ARG);
  CHECK(ssp_ranst_piecewise_linear_setup
    (pwl, intervals, weights, 1), RES_BAD_ARG);
  CHECK(ssp_ranst_piecewise_linear_setup
    (pwl, intervals, weights, sizeof(intervals)/sizeof(double)), RES_OK);

  CHECK(ssp_ranst_piecewise_linear_ref_get(NULL), RES_BAD_ARG);
  CHECK(ssp_ranst_piecewise_linear_ref_get(pwl), RES_OK);

  CHECK(ssp_ranst_piecewise_linear_ref_put(NULL), RES_BAD_ARG);
  CHECK(ssp_ranst_piecewise_linear_ref_put(pwl), RES_OK);

  FOR_EACH(i, 0, NBS) {
    double r = ssp_ranst_piecewise_linear_get(pwl, rng);
    CHECK(0 <= r && r <= 10, 1);
    x += r;
    x2 += r * r;
  }

  mean = x/NBS;
  std = sqrt(x2/NBS - mean*mean);
  printf("%g %g\n", mean, std);
  CHECK(eq_eps(mean, exp_mean, 1e-2), 1);
  CHECK(eq_eps(std, exp_std, 1.e-2), 1);

  CHECK(ssp_ranst_piecewise_linear_ref_put(pwl), RES_OK);

  CHECK(ssp_rng_ref_put(rng), RES_OK);

  check_memory_allocator(&allocator);
  mem_shutdown_proxy_allocator(&allocator);
  CHECK(mem_allocated_size(), 0);
  return 0;
}
